(function ($, w, d) {

    if (!w.FiloBlu) {
        w.FiloBlu = {};
    }

    w.FiloBlu.Utils = {
        options: {},
        init: function (params) {
            var self = this;
            self.options = $.extend(self.options, params);

            if (typeof (Modernizr) === "undefined") {
                self.isTouchDevice = (typeof (self.isTouchDevice) !== "undefined") ? self.isTouchDevice : self.checkIsTouch();
            } else {
                self.isTouchDevice = Modernizr.touch;
            }

            // INIT
            self.eventsOnChangeVieport();
            self.setScreenType(true);
            //self.resizeVideoJS();

            //resize
            self.windowResize((function () {
                self.setScreenType(false);
            }), 0);
            
            //video
            if ( self.options.videojs_4_12_7 ) {

                self.resizeVideoJS();

                self.windowResize((function () {
                    self.resizeVideoJS();
                }), 300);
            }
            
            
              requirejs(["domReady!"], function () {
                  
                  self.setScreenType(true);
                  self.replaceTextTruncate();
                  
                   //lazyload
                    if ( self.options.lazyload ) {
                        self.responsiveImagesDynamicHeight();

                        self.windowResize((function () {
                            self.responsiveImagesDynamicHeight();
                        }), 0);
                    }
                    //rwdImageMaps
                    if (self.options.rwdImageMaps_1_5) {
                        requirejs(["rwdImageMaps_1_5"], function () {

                            $('img[usemap]').rwdImageMaps();

                            self.windowResize((function () {
                                $('img[usemap]').rwdImageMaps();
                            }), 300);

                            $(w).load(function () {
                                $('img[usemap]').rwdImageMaps();
                            });

                        });
                    }
                });

        },
        isValidMediaQuery: function (media) {
            // MatchMedia support detection.
            if ("matchMedia" in window) {
                if (window.matchMedia(media).matches) {
                    return true;
                } else {
                    return false;
                }
            }
            // Fallback using Modernizr MatchMedia support detection.
            else if (Modernizr) { // Prevent error if Modernizr isn't availabe.
                if (Modernizr.mq(media)) {
                    return true;
                } else {
                    return false;
                }
            }
            // Return false if mediaquery detection is unavailable.
            return false;
        },
        windowResize: function (functionExec, timeout) {
            var resizeTimer;
            if (!timeout) {
                timeout = 100;
            }
            resizeTimer = w.setTimeout(function () {
                null;
            }, 0);
            $(w).resize(function () {
                w.clearTimeout(resizeTimer);
                resizeTimer = w.setTimeout(functionExec, timeout);
            });
        },
        setScreenType: function (force) {

            var self        = this,
                eventName   = "";

            //loop all the breakpoint
            $.each(self.options.breakpoint, function (key, val) {

                self['screenIs' + val[0]] = false;

                if (
                        (self.oldBreakPointType != val[0] || force) &&
                        self.isValidMediaQuery( val[1] )
                    ) {

                    self['screenIs' + val[0]] = true;
                    self.oldBreakPointType = val[0];
                    eventName = "screen.is." + val[0];
                }
            });

            if (eventName) {
                $(w).trigger(eventName);
            }

        },
        getAreaCenter: function (shape, coords) {
            var center, coord, coordsArray, i, l, maxX, maxY, minX, minY;

            coordsArray = coords.split(',');
            center = [];
            if (shape === 'circle') {
                center = [coordsArray[0], coordsArray[1]];
            } else {
                coord = void 0;
                minX = maxX = parseInt(coordsArray[0], 10);
                minY = maxY = parseInt(coordsArray[1], 10);
                i = 0;
                l = coordsArray.length;
                while (i < l) {
                    coord = parseInt(coordsArray[i], 10);
                    if (i % 2 === 0) {
                        if (coord < minX) {
                            minX = coord;
                        } else if (coord > maxX) {
                            maxX = coord;
                        }
                    } else {
                        if (coord < minY) {
                            minY = coord;
                        } else if (coord > maxY) {
                            maxY = coord;
                        }
                    }
                    i++;
                }
                center = [parseInt((minX + maxX) / 2, 10), parseInt((minY + maxY) / 2, 10)];
            }
            return center;
        },
        checkIsTouch: function () {
            var el, isMobileDevice;

            isMobileDevice = /Android|webOS|iPhone|iPad|iPod|IEMobile|BlackBerry/i.test(navigator.userAgent);
            el = document.createElement("div");
            el.setAttribute("ongesturestart", "return;");
            return isMobileDevice || typeof el.ongesturestart === "function";
        },
        resizeVideoJS: function () {

            $('.video-js.resize-video').each(function () {
                var aspectRatio,
                        jsPlayer,
                        widthPlayer,
                        x,
                        y;

                jsPlayer = $(this);
                x = jsPlayer.attr("data-resize-x");
                y = jsPlayer.attr("data-resize-y");


                if (typeof x === "undefined") {
                    x = 16;
                }
                if (typeof y === "undefined") {
                    y = 9;
                }

                aspectRatio = y / x;

                if (jsPlayer.length > 0) {
                    widthPlayer = jsPlayer.parent().outerWidth();

                    return jsPlayer.css('width', widthPlayer + 'px').css('height', (widthPlayer * aspectRatio) + 'px');
                }

            });

        },
        facebook_share: function (appId, urlToShare, titleToShare, DescToShare, imageToShare) {
            w.open("https://www.facebook.com/dialog/feed?app_id=" + appId + "&display=popup&caption=" + encodeURIComponent(titleToShare) + "&description=" + encodeURIComponent(DescToShare) + "&link=" + encodeURIComponent(urlToShare) + "&picture=" + encodeURIComponent(imageToShare) + "&redirect_uri=" + encodeURIComponent(urlToShare), "fb_sharer", "toolbar=0,status=0,width=626,height=436");
            return false;
        },
        twitter_share: function (urlToShare, statusToShare) {
            w.open("http://twitter.com/home?status=" + encodeURIComponent(statusToShare) + " " + encodeURIComponent(urlToShare), "tw_sharer", "toolbar=0,status=0,width=626,height=436");
            return false;
        },
        pinterest_share: function (urlToShare, titleToShare, imageToShare) {
            w.open("http://pinterest.com/pin/create/button/?url=" + encodeURIComponent(urlToShare) + "&media=" + encodeURIComponent(imageToShare) + "&description=" + encodeURIComponent(titleToShare), "pinterest-share-dialog", "toolbar=0,status=0,width=626,height=436");
            return false;
        },
        replaceTextTruncate: function (elementSelector) {
            elementSelector = elementSelector || '.truncate-text';
            var self,
                    truncatedElements,
                    originalString,
                    toTruncate,
                    truncatedString;

            self = this;
            truncatedElements = $(elementSelector);

             if (truncatedElements.length) {
                truncatedElements.each(function () {

                    toTruncate = $(this);
                    originalString = toTruncate.data("original-string");
                    truncatedString = '';

                    truncatedString = self.truncateText(originalString, toTruncate.data('max-char-' + self.oldBreakPointType));

                    return toTruncate.html(truncatedString);
                });
            }
        },
        truncateText: function (string, max_char) {
            var a,
                    strpos,
                    trimmed = string.trim();

            strpos = function (haystack, needle, offset) {
                var a,
                        i = (haystack + '').indexOf(needle, offset || 0);

                if (i === -1) {
                    return false;
                } else {
                    return i;
                }
            };

            if (trimmed.length > max_char) {
                a = trimmed.substr(0, strpos(trimmed, ' ', max_char)) + '...';
                if (a === '...') {
                    a = string;
                }
            } else {
                a = string;
            }

            return a;
        },
        eventsOnChangeVieport: function () {

            var self        = this,
                eventsName  = "";

            //loop all the breakpoint
            $.each(self.options.breakpoint, function (key, val) {

                eventsName += " screen.is." + val[0];
                
            });

            if (eventsName) {
                $(w).on(eventsName, function (event) {
                    self.replaceTextTruncate();
                });
            }

        },
        responsiveImagesDynamicHeight: function() {
            var self        = this,
                childImg,
                coeff,
                originalHeight,
                originalWidth;

            $('.lazyload-container').each(function ( index ) {

                childImg = $('>img.lazyload', $(this));

                //loop all the breakpoint
                $.each(self.options.breakpoint, function (key, val) {

                    if (self.isValidMediaQuery(val[1])) {
                        originalWidth = childImg.data('original-width-' + val[0]);
                        originalHeight = childImg.data('original-height-' + val[0]);

                    }
                });

                if (!originalWidth || !originalHeight) {
                    originalWidth = childImg.data('original-width');
                    originalHeight = childImg.data('original-height');
                }


                if (originalWidth && originalHeight) {
                    coeff = (originalHeight / originalWidth) * 100;

                    $(this).css({
                        'position'          : 'relative',
                        'height'            : 0,
                        'padding-bottom'    : parseFloat(coeff).toFixed(1) + '%'
                    });
                    childImg.css({
                        'position'          : 'absolute',
                        'top'               : 0,
                        'left'              : 0,
                        'width'             : '100%',
                        'height'            : '100%'
                    });
                }
            });
        },
        /**
         * @param  {[String]}   formId                      [ form DOM selector ]
         * @param  {[String]}   messageContainerId          [ message container DOM selector ]
         * @param  {[String]}   messageContainerSuccessId   [ message container DOM selector ]
         * @param  {[String]}   messageContainerFailedId    [ message container DOM selector ]
         * @param  {[String]}   btnToDisable                [ submit button DOM selector ]
         * @param  {[function]} functionCallbackBefore      [ function call when server respond ]
         * @param  {[function]} functionCallbackAfter       [ function call when server ]
         * @param  {[function]} functionCallbackOnSuccess   [ function call if the ajax request success ]
         * @param  {[function]} functionCallbackOnFailed    [ function call if the ajax request faild ]
         * @param  {[boolean]} useWrapper                   [ used for adding the message wrapper ]
         * @param  {[function]} functionBeforeAll           [ function call before all when server respond  ]
         */
        requestAjaxForm : function( formId, messageContainerId, messageContainerSuccessId, messageContainerFailedId, btnToDisable, functionCallbackBefore, functionCallbackAfter, functionCallbackOnSuccess, functionCallbackOnFailed,  useWrapper, functionBeforeAll ) {

            if( !$(formId).valid() ){
                return;
            }

            var submitForm      = $(formId),
                wrapAfter       = useWrapper ? '</div>' : '',
                wrapBefore      = useWrapper ? '<div class="page messages">' : '',
                formData        = {},
                toggleButton    = function( action ){
                                    if( btnToDisable ) {
                                        $(btnToDisable).prop( "disabled", action );
                                    }
                                },
                isFunction      = function( fn ){
                                    return ( typeof( fn ) === 'function' ) ? true : false;
                                };

            if ( isFunction( functionBeforeAll ) ) {
                functionBeforeAll();
            }

            if ( messageContainerId.length > 0 ) {
                $(messageContainerId).html('');
            }
            if ( messageContainerSuccessId.length > 0 ) {
                $(messageContainerSuccessId).html('');
            }
            if ( messageContainerFailedId.length > 0 ) {
                $(messageContainerFailedId).html('');
            }

            toggleButton( true );

            submitForm.serializeArray().map( function( item ){
                formData[ item.name ] = item.value;
            });

            $.ajax({
                method      : 'POST',
                url         : submitForm.attr('action'),
                data        : formData
            }).done(function( response ) {

                if( response.responseText ){

                    var result              = JSON.parse(response.responseText),
                        responseHtmlMessage = function( messageStyle, message ){
                                                return wrapBefore + '<div class="' + messageStyle + ' message"><div>' + message + '</div></div>' + wrapAfter;
                                            };

                    if ( isFunction( functionCallbackBefore ) ) {
                        functionCallbackBefore();
                    }

                    switch ( result.responseType ) {
                        case 1:
                            if ($(messageContainerId).length > 0) {
                                $(messageContainerId).html( responseHtmlMessage( 'message-success success ', result["message"] ) );
                            }
                            if ($(messageContainerSuccessId).length > 0) {
                                $(messageContainerSuccessId).html( responseHtmlMessage( 'message-success success ', result["message"] ) );
                            }
                            if ( isFunction( functionCallbackOnSuccess ) ) {
                                functionCallbackOnSuccess( result );
                            }

                            break;
                        case 2:
                            if ($(messageContainerId).length > 0) {
                                $(messageContainerId).html( responseHtmlMessage( 'message-error error', result["message"] ) );
                            }
                            if ($(messageContainerFailedId).length > 0) {
                                $(messageContainerFailedId).html( responseHtmlMessage( 'message-error error', result["message"] ) );
                            }
                            if ( isFunction( functionCallbackOnFailed ) ) {
                                functionCallbackOnFailed(result);
                            }
                            break;
                        case 3:
                            if ($(messageContainerId).length > 0) {
                                $(messageContainerId).html( responseHtmlMessage( 'message-notice notice', result["message"] ) );
                            }
                            break;
                    }


                    if (result["redirect"]) {
                        if (result["redirect"] === '/') {
                            w.location.reload();
                        } else {
                            w.location.href = result["redirect"];
                        }
                    }
                }

                toggleButton( false );

                if ( isFunction( functionCallbackAfter ) ) {
                    functionCallbackAfter(result);
                }

            });

        }
    };

})(jQuery, window, document);
